@@ -87,6 +87,7 @@ func CmdName() string {
 }
 
 func main() {
+	forcedIfchange := flag.Bool("f", false, "forced redo-ifchange")
 	version := flag.Bool("version", false, "print version")
 	warranty := flag.Bool("warranty", false, "print warranty information")
 
@@ -335,7 +336,7 @@ CmdSwitch:
 			}
 		}
 	case CmdNameRedoIfchange:
-		ok, err = ifchange(tgts, false, traced)
+		ok, err = ifchange(tgts, *forcedIfchange, traced)
 		if err == nil {
 			err = depsWrite(fdDep, tgts)
 		}
